var beaconScanner = new BeaconScanner();

function BeaconScanner() {
   var Beacons = [];
   var device = IR.CreateDevice(IR.DEVICE_BLE_SCANNER);

   this.Get = function(in_mac) {
      for(var i = 0; i < Beacons.length; i++) {
         if(Beacons[i].MAC == in_mac) {
            return Beacons[i];
         }
      }
      return null;   
   }
   
   this.GetByPos = function(in_pos) {
      return Beacons[in_pos];
   }
   
   function BLEreceive(in_mac, in_name, in_rssi, in_advertisementData) {
      // search for existing Beacon
      if(in_rssi < 0) {
         var temp = this.Get(in_mac);
         if(temp) {
            temp.Update.call(temp, in_rssi);
         } else {
            var temp_beacon = new Beacon(in_name, in_mac, in_rssi);
            Beacons.push(temp_beacon);
            gui.ListPush(temp_beacon);
            IR.Log("Beacon finded: " + in_name + ", MAC: " + in_mac);      
         }
         gui.UpdateCurrent();      
      }
   }
   
   IR.AddListener(IR.EVENT_DEVICE_BLE_FOUND, 0, BLEreceive, this);
   
   device.Connect();
}

function Beacon(in_name, in_mac, in_rssi, in_tx_power) {
   this.MAC = in_mac;
   this.Name = in_name;
   this.CurrentRSSI = in_rssi;
   this.AverageRSSI = in_rssi;
   this.Distance = -1;
   this.CalibrationStatus = 0;
   
   if(in_tx_power) {
      this.CalibrationStatus = 1;
      this.TXPower = in_tx_power;
   } else {
      this.CalibrationStatus = 0;
      this.TXPower = 0;
   }
   
   var RSSI_History = [in_rssi];

	function HistoryPush(in_rssi) {
		if(in_rssi < 0) {
			if(RSSI_History.length == 20) {
				RSSI_History.shift();
				RSSI_History.push(in_rssi);
			} else {
				RSSI_History.push(in_rssi);
			}
		}
	}
	
	function AverageUpdate() {
		var summ = 0;
		for(var i = 0; i < RSSI_History.length; i++) {
			summ += RSSI_History[i];
		}
		var ave = summ/RSSI_History.length;
      
      this.AverageRSSI = ave;
      
      var t_dist = getDistance.call(this, ave);
   	if (t_dist != -1) {
   		this.Distance = t_dist;
   	}         
	}
	
	function getDistance(in_rssi) { 
		if (in_rssi > 0) {
			return -1;
		}
		if (!this.CalibrationStatus) {
			return -1; // 
		}
		var ratio = in_rssi * 1.0 / this.TXPower;
		if (ratio < 1.0) {
			return parseFloat(Math.pow(ratio, 10).toFixed(1))
		} else {
			return parseFloat((0.89976 * Math.pow(ratio, 7.7095) + 0.111).toFixed(1))
		}
	}
   
   this.Update = function(in_rssi) {
      this.CurrentRSSI = in_rssi;
      HistoryPush(in_rssi);
      AverageUpdate.call(this);
   }
   
   this.Calibrate = function() {
		var RSSIArray = [];
      IR.Log("Calibration started");

		var checkInterval = IR.SetInterval(500, function() {
			RSSIArray.push(this.CurrentRSSI);
			if(RSSIArray.length == 40) {
				var average = 0;
				for(var i = 0; i < RSSIArray.length; i++) {
					average += RSSIArray[i];
				}
				average /= RSSIArray.length;
            this.CalibrationStatus = 1;
            this.TXPower = average;
            IR.Log("Calibration completed");
			}
		}, this);
   }
}
